@@ -1,15 +1,11 @@
-import re
-import warnings
-from copy import deepcopy
 from typing import Dict, List
 
-from pypika import Criterion, JoinType, Order, Table
+from pypika import Criterion, Order, Table
 from pypika.terms import LiteralValue
 
-from metrics_layer.core.model.base import MetricsLayerBase
+from metrics_layer.core.sql.query_base import MetricsLayerQueryBase
 from metrics_layer.core.model.definitions import Definitions
 from metrics_layer.core.model.field import Field
-from metrics_layer.core.model.join import Join
 from metrics_layer.core.model.view import View
 from metrics_layer.core.model.filter import LiteralValueCriterion
 from metrics_layer.core.sql.query_design import MetricsLayerDesign
@@ -18,7 +14,7 @@ from metrics_layer.core.sql.query_errors import ArgumentError
 from metrics_layer.core.sql.query_filter import MetricsLayerFilter
 
 
-class MetricsLayerQuery(MetricsLayerBase):
+class MetricsLayerQuery(MetricsLayerQueryBase):
     """ """
 
     def __init__(self, definition: Dict, design: MetricsLayerDesign, suppress_warnings: bool = False) -> None:
@@ -32,6 +28,7 @@ class MetricsLayerQuery(MetricsLayerBase):
         # A collection of all the column and aggregate filters in the query + order by
         self.where_filters = []
         self.having_filters = []
+        self.having_group_by_filters = []
         self.order_by_args = []
 
         self.parse_definition(definition)
@@ -44,15 +41,11 @@ class MetricsLayerQuery(MetricsLayerBase):
         having = definition.get("having", None)
         order_by = definition.get("order_by", None)
 
-        always_where = self.design.explore.sql_always_where
         access_filter_literal, _ = self.design.get_access_filter()
-        if where or always_where or access_filter_literal:
-            wheres = where
-            self.where_filters.extend(
-                self._parse_filter_object(
-                    wheres, "where", always_where=always_where, access_filter=access_filter_literal
-                )
-            )
+        if where or access_filter_literal:
+            wheres = self._parse_filter_object(where, "where", access_filter=access_filter_literal)
+            self.where_filters.extend([f for f in wheres if not f.is_group_by])
+            self.having_group_by_filters.extend([f for f in wheres if f.is_group_by])
 
         if having and self.no_group_by:
             raise ArgumentError(
@@ -61,7 +54,9 @@ class MetricsLayerQuery(MetricsLayerBase):
             )
 
         if having:
-            self.having_filters.extend(self._parse_filter_object(having, "having"))
+            having_filters = self._parse_filter_object(having, "having")
+            self.having_group_by_filters.extend([f for f in having_filters if f.is_group_by])
+            self.having_filters.extend([f for f in having_filters if not f.is_group_by])
 
         if order_by and self.no_group_by:
             raise ArgumentError(
@@ -75,26 +70,10 @@ class MetricsLayerQuery(MetricsLayerBase):
         elif self.query_type in {Definitions.snowflake, Definitions.redshift}:
             self.order_by_args.append({"field": "__DEFAULT__"})
 
-    def _parse_filter_object(
-        self, filter_object, filter_type: str, always_where: str = None, access_filter: str = None
-    ):
+    def _parse_filter_object(self, filter_object, filter_type: str, access_filter: str = None):
         results = []
         extra_kwargs = dict(filter_type=filter_type, design=self.design)
 
-        always_where_is_valid = always_where and "{%" not in always_where
-        if always_where_is_valid:
-            filter_literal = MetricsLayerFilter(definition={"literal": always_where}, **extra_kwargs)
-            results.append(filter_literal)
-
-        if always_where and not always_where_is_valid and not self.suppress_warnings:
-            warnings.warn(
-                (
-                    "Always where clause NOT being applied due to "
-                    f"Looker conditional in the clause (to suppress these warnings pass"
-                    f" the argument 'suppress_warnings=True' to the function):\n\n {always_where}"
-                )
-            )
-
         if access_filter:
             filter_literal = MetricsLayerFilter(definition={"literal": access_filter}, **extra_kwargs)
             results.append(filter_literal)
@@ -135,7 +114,7 @@ class MetricsLayerQuery(MetricsLayerBase):
         return len(self.design.joins()) > 0
 
     def get_query(self, semicolon: bool = True):
-        # Build the base_join table if of join is needed otherwise use a single table
+        # Build the base_join table if a join is needed otherwise use a single table
         if self.needs_join():
             base_query = self.get_join_query_from()
         else:
@@ -160,6 +139,15 @@ class MetricsLayerQuery(MetricsLayerBase):
             having = [f.sql_query() for f in self.having_filters]
             base_query = base_query.having(Criterion.all(having))
 
+        if self.having_group_by_filters:
+            group_by_where = []
+            for i, f in enumerate(sorted(self.having_group_by_filters)):
+                cte_alias = f"filter_subquery_{i}"
+                cte_query = f.cte(query_class=MetricsLayerQuery, design_class=MetricsLayerDesign)
+                base_query = base_query.with_(Table(cte_query), cte_alias)
+                group_by_where.append(f.group_by_sql_query(cte_alias=cte_alias, query_generator=self))
+            base_query = base_query.where(Criterion.all(group_by_where))
+
         # Handle order by
         if self.order_by_args and not self.no_group_by:
             for arg in self.order_by_args:
@@ -238,7 +226,7 @@ class MetricsLayerQuery(MetricsLayerBase):
     # Code to handle the FROM portion of the query
     def get_join_query_from(self):
         # Build the base_join table
-        base_join_query = self._get_base_query()
+        base_join_query = self._base_query()
 
         # Base table from statement
         table = self.design.get_view(self.design.base_view_name)
@@ -247,51 +235,38 @@ class MetricsLayerQuery(MetricsLayerBase):
 
         # Start By building the Join
         for join in self.design.joins():
-            table = self.design.get_view(join.from_)
+            table = self.design.get_view(join.join_view_name)
 
             # Create a pypika Table based on the Table's name
             db_table = self._table_expression(table)
-            if isinstance(db_table, str):
-                db_table = Table(db_table)
-                # raise NotImplementedError("TODO: handle sub queries (derived tables for joins)")
 
             criteria = LiteralValueCriterion(join.replaced_sql_on(self.query_type))
             join_type = self.get_pypika_join_type(join)
 
-            base_join_query = base_join_query.join(db_table, join_type).on(criteria)
+            if join.type == "cross":
+                base_join_query = base_join_query.join(db_table, join_type).cross()
+            else:
+                base_join_query = base_join_query.join(db_table, join_type).on(criteria)
 
         return base_join_query
 
     def get_single_table_query_from(self):
-        base_query = self._get_base_query()
+        base_query = self._base_query()
 
         table = self.design.get_view(self.design.base_view_name)
         base_query = base_query.from_(self._table_expression(table))
 
         return base_query
 
-    def _get_base_query(self):
-        return self.query_lookup[self.query_type]
-
     def _table_expression(self, view: View):
         # Create a pypika Table based on the Table's name or it's derived table sql definition
         if view.derived_table:
             derived_sql = view.derived_table["sql"]
-            table_expr = f"({derived_sql}) as {view.name}"
+            table_expr = Table(f"({derived_sql}) as {view.name}")
         else:
             table_expr = Table(view.sql_table_name, alias=view.name)
         return table_expr
 
-    @staticmethod
-    def get_pypika_join_type(join: Join):
-        if join.type == "left_outer":
-            return JoinType.left
-        elif join.type == "inner":
-            return JoinType.inner
-        elif join.type == "full_outer":
-            return JoinType.outer
-        return JoinType.left
-
     # Code for the GROUP BY part of the query
     def get_group_by_columns(self):
         group_by = []
@@ -310,20 +285,3 @@ class MetricsLayerQuery(MetricsLayerBase):
         else:
             query = field.sql_query(query_type=self.query_type)
         return self.sql(query, alias)
-
-    @staticmethod
-    def sql(sql: str, alias: str = None):
-        if alias:
-            return LiteralValue(sql + f" as {alias}")
-        return LiteralValue(sql)
-
-    @staticmethod
-    def strip_alias(sql: str):
-        stripped_sql = deepcopy(sql)
-        matches = re.findall(r"(?i)\ as\ ", stripped_sql)
-        if matches:
-            alias = " AS "
-            for match in matches:
-                stripped_sql = stripped_sql.replace(match, alias)
-            return alias.join(stripped_sql.split(alias)[:-1])
-        return sql
